@@ -13,35 +13,29 @@
 ##############################################################################
 
 from zope.interface import Interface, Attribute
+from zope.interface.common.collections import ISized
+from zope.interface.common.sequence import IMinimalSequence
+from zope.interface.common.collections import IMapping
 
+# pylint:disable=inherit-non-class,no-method-argument,no-self-argument
+# pylint:disable=unexpected-special-method-signature
 
 class ICollection(Interface):
+    """
+    A collection of zero or more objects.
+
+    In a boolean context, objects implementing this interface are
+    `True` if the collection is non-empty, and `False` if the
+    collection is empty.
+    """
 
     def clear():
         """Remove all of the items from the collection."""
 
-    def __nonzero__():
-        """Check if the collection is non-empty.
 
-        Return a true value if the collection is non-empty and a
-        false value otherwise.
-        """
-
-
-class IReadSequence(Interface):
-
-    def __getitem__(index):
-        """Return the value at the given index.
-
-        An IndexError is raised if the index cannot be found.
-        """
-
-    def __getslice__(index1, index2):
-        """Return a subsequence from the original sequence.
-
-        The subsequence includes the items from index1 up to, but not
-        including, index2.
-        """
+# Backwards compatibility alias. To be removed in 5.0.
+# Docs deprecated only in docs/api.rst.
+IReadSequence = IMinimalSequence
 
 class IKeyed(ICollection):
 
@@ -52,25 +46,29 @@ class IKeyed(ICollection):
         """
 
     def keys(min=None, max=None, excludemin=False, excludemax=False):
-        """Return an IReadSequence containing the keys in the collection.
+        """
+        Return an :mod:`IMinimalSequence <zope.interface.common.sequence>`
+        containing the keys in the collection.
 
-        The type of the IReadSequence is not specified.  It could be a list
-        or a tuple or some other type.
+        The type of the ``IMinimalSequence`` is not specified. It
+        could be a `list` or a `tuple` or some other type.
 
         All arguments are optional, and may be specified as keyword
         arguments, or by position.
 
-        If a min is specified, then output is constrained to keys greater
-        than or equal to the given min, and, if excludemin is specified and
-        true, is further constrained to keys strictly greater than min.  A
-        min value of None is ignored.  If min is None or not specified, and
-        excludemin is true, the smallest key is excluded.
+        If a *min* is specified, then output is constrained to keys
+        greater than or equal to the given min, and, if *excludemin*
+        is specified and true, is further constrained to keys strictly
+        greater than *min*. A *min* value of `None` is ignored. If
+        *min* is `None` or not specified, and *excludemin* is true,
+        the smallest key is excluded.
 
-        If a max is specified, then output is constrained to keys less than
-        or equal to the given max, and, if excludemax is specified and
-        true, is further constrained to keys strictly less than max.  A max
-        value of None is ignored.  If max is None or not specified, and
-        excludemax is true, the largest key is excluded.
+        If a *max* is specified, then output is constrained to keys
+        less than or equal to the given *max*, and, if *excludemax* is
+        specified and true, is further constrained to keys strictly
+        less than *max*. A *max* value of `None` is ignored. If *max*
+        is `None` or not specified, and *excludemax* is true, the
+        largest key is excluded.
         """
 
     def maxKey(key=None):
@@ -101,19 +99,88 @@ class ISetMutable(IKeyed):
     def remove(key):
         """Remove the key from the set.
 
-        Raises KeyError if key is not in the set.
+        Raises :class:`KeyError` if key is not in the set.
         """
 
     def update(seq):
         """Add the items from the given sequence to the set."""
 
+    def __and__(other):
+        """
+        Shortcut for :meth:`~BTrees.Interfaces.IMerge.intersection`
 
-class ISized(Interface):
-    """An object that supports __len__."""
+        .. versionadded:: 4.8.0
+        """
 
-    def __len__():
-        """Return the number of items in the container."""
+    def __iand__(other):
+        """
+        As for :meth:`set.intersection_update`: Update this object,
+        keeping only elements found in both it and other.
 
+        .. versionadded:: 4.8.0
+        """
+
+    def __or__(other):
+        """
+        Shortcut for :meth:`~BTrees.Interfaces.IMerge.union`
+
+        .. versionadded:: 4.8.0
+        """
+
+    def __ior__(other):
+        """
+        As for :meth:`set.update`: Update this object, adding
+        elements from *other*.
+
+        .. versionadded:: 4.8.0
+        """
+
+    def __sub__(other):
+        """
+        Shortcut for :meth:`~BTrees.Interfaces.IMerge.difference`
+
+        .. versionadded:: 4.8.0
+        """
+
+    def __isub__(other):
+        """
+        As for :meth:`set.difference_update`: Update this object,
+        removing elements found in *other*.
+
+        .. versionadded:: 4.8.0
+        """
+
+    def isdisjoint(other):
+        """
+        As for :meth:`set.isdisjoint`: Return True if the set has no
+        elements in common with other.
+
+        .. versionadded:: 4.8.0
+        """
+
+    def discard(key):
+        """
+        As for :meth:`set.discard`: Remove the *key* from the set,
+        but only if it is present.
+
+        .. versionadded:: 4.8.0
+        """
+
+    def pop():
+        """
+        As for :meth:`set.pop`: Remove and return an arbitrary element;
+        raise :exc:`KeyError` if the object is empty.
+
+        .. versionadded:: 4.8.0
+        """
+
+    def __ixor__(other):
+        """
+        As for :meth:`set.symmetric_difference_update`: Update this object,
+        keeping only elements found in either set but not in both.
+
+        .. versionadded:: 4.8.0
+        """
 
 class IKeySequence(IKeyed, ISized):
 
@@ -125,25 +192,37 @@ class IKeySequence(IKeyed, ISized):
         """
 
 
-class ISet(IKeySequence, ISetMutable):
-    pass
+class ISet(ISetMutable, IKeySequence):
+    """
+    A set of unique items stored in a single persistent object.
+    """
+
+class ITreeSet(ISetMutable):
+    """
+    A set of unique items stored in a tree of persistent objects.
+    """
 
 
-class ITreeSet(IKeyed, ISetMutable):
-    pass
+class IMinimalDictionary(IKeyed, IMapping):
+    """
+    Mapping operations.
 
-class IMinimalDictionary(ISized, IKeyed):
+    .. versionchanged:: 4.8.0
+       Now extends :class:`zope.interface.common.collections.IMapping`.
+    """
 
     def get(key, default):
         """Get the value associated with the given key.
 
-        Return the default if has_key(key) is false.
+        Return the default if :meth:`~BTrees.Interfaces.IKeyed.has_key` is false
+        with the given key.
         """
 
     def __getitem__(key):
         """Get the value associated with the given key.
 
-        Raise KeyError if has_key(key) is false.
+        Raise :class:`KeyError` if :meth:`~BTrees.Interfaces.IKeyed.has_key` is false
+        with the given key.
         """
 
     def __setitem__(key, value):
@@ -152,57 +231,66 @@ class IMinimalDictionary(ISized, IKeyed):
     def __delitem__(key):
         """Delete the value associated with the given key.
 
-        Raise KeyError if has_key(key) is false.
+        Raise class:`KeyError` if :meth:`~BTrees.Interfaces.IKeyed.has_key` is false
+        with the given key.
         """
 
     def values(min=None, max=None, excludemin=False, excludemax=False):
-        """Return an IReadSequence containing the values in the collection.
+        """
+        Return an :mod:`IMinimalSequence <zope.interface.common.sequence.IMinimalSequence>`
+        containing the values in the collection.
 
-        The type of the IReadSequence is not specified.  It could be a list
-        or a tuple or some other type.
+        The type of the ``IMinimalSequence`` is not specified. It
+        could be a `list` or a `tuple` or some other type.
 
         All arguments are optional, and may be specified as keyword
         arguments, or by position.
 
-        If a min is specified, then output is constrained to values whose
-        keys are greater than or equal to the given min, and, if excludemin
-        is specified and true, is further constrained to values whose keys
-        are strictly greater than min.  A min value of None is ignored.  If
-        min is None or not specified, and excludemin is true, the value
-        corresponding to the smallest key is excluded.
+        If a *min* is specified, then output is constrained to values
+        whose keys are greater than or equal to the given *min*, and, if
+        *excludemin* is specified and true, is further constrained to
+        values whose keys are strictly greater than *min*. A *min* value
+        of `None` is ignored. If *min* is `None` or not specified, and
+        *excludemin* is true, the value corresponding to the smallest
+        key is excluded.
 
-        If a max is specified, then output is constrained to values whose
-        keys are less than or equal to the given max, and, if excludemax is
-        specified and true, is further constrained to values whose keys are
-        strictly less than max.  A max value of None is ignored.  If max is
-        None or not specified, and excludemax is true, the value
-        corresponding to the largest key is excluded.
+        If a *max* is specified, then output is constrained to values
+        whose keys are less than or equal to the given *max*, and, if
+        *excludemax* is specified and true, is further constrained to
+        values whose keys are strictly less than *max*. A *max* value of
+        `None` is ignored. If *max* is `None` or not specified, and
+        *excludemax* is true, the value corresponding to the largest key
+        is excluded.
         """
 
     def items(min=None, max=None, excludemin=False, excludemax=False):
-        """Return an IReadSequence containing the items in the collection.
+        """
+        Return an ``IMinimalSequence`` containing the items in the
+        collection.
 
-        An item is a 2-tuple, a (key, value) pair.
+        An item is a 2-tuple, a ``(key, value)`` pair.
 
-        The type of the IReadSequence is not specified.  It could be a list
-        or a tuple or some other type.
+        The type of the ``IMinimalSequence`` is not specified. It
+        could be a `list` or a `tuple` or some other type.
 
         All arguments are optional, and may be specified as keyword
         arguments, or by position.
 
-        If a min is specified, then output is constrained to items whose
-        keys are greater than or equal to the given min, and, if excludemin
-        is specified and true, is further constrained to items whose keys
-        are strictly greater than min.  A min value of None is ignored.  If
-        min is None or not specified, and excludemin is true, the item with
-        the smallest key is excluded.
+        If a *min* is specified, then output is constrained to items
+        whose keys are greater than or equal to the given *min*, and,
+        if *excludemin* is specified and true, is further constrained
+        to items whose keys are strictly greater than *min*. A *min*
+        value of `None` is ignored. If *min* is `None` or not
+        specified, and *excludemin* is true, the item with the
+        smallest key is excluded.
 
-        If a max is specified, then output is constrained to items whose
-        keys are less than or equal to the given max, and, if excludemax is
-        specified and true, is further constrained to items whose keys are
-        strictly less than max.  A max value of None is ignored.  If max is
-        None or not specified, and excludemax is true, the item with the
-        largest key is excluded.
+        If a *max* is specified, then output is constrained to items
+        whose keys are less than or equal to the given *max*, and, if
+        *excludemax is specified and true, is further constrained to
+        items whose keys are strictly less than *max*. A *max* value
+        of `None` is ignored. If *max* is `None` or not specified, and
+        *excludemax* is true, the item with the largest key is
+        excluded.
         """
 
 class IDictionaryIsh(IMinimalDictionary):
@@ -226,22 +314,30 @@ class IDictionaryIsh(IMinimalDictionary):
     def setdefault(key, d):
         """D.setdefault(k, d) -> D.get(k, d), also set D[k]=d if k not in D.
 
-        Return the value like get() except that if key is missing, d is both
+        Return the value like :meth:`~BTrees.Interfaces.IMinimalDictionary.get` except that if key is missing, d is both
         returned and inserted into the dictionary as the value of k.
 
-        Note that, unlike as for Python's dict.setdefault(), d is not
+        Note that, unlike as for Python's :meth:`dict.setdefault`, d is not
         optional.  Python defaults d to None, but that doesn't make sense
         for mappings that can't have None as a value (for example, an
-        IIBTree can have only integers as values).
+        :class:`~BTrees.IIBTree.IIBTree` can have only integers as values).
         """
 
     def pop(key, d):
         """D.pop(k[, d]) -> v, remove key and return the corresponding value.
 
-        If key is not found, d is returned if given, otherwise KeyError is
+        If key is not found, d is returned if given, otherwise :class:`KeyError` is
         raised.
         """
 
+    def popitem():
+        """
+        D.popitem() -> (k, v), remove and return some (key, value) pair
+        as a 2-tuple; but raise KeyError if D is empty.
+
+        .. versionadded:: 4.8.0
+        """
+
 
 class IBTree(IDictionaryIsh):
 
@@ -264,6 +360,15 @@ class IBTree(IDictionaryIsh):
               key=generate_key()
         """
 
+    def __and__(other):
+        """Shortcut for :meth:`~BTrees.Interfaces.IMerge.intersection`"""
+
+    def __or__(other):
+        """Shortcut for :meth:`~BTrees.Interfaces.IMerge.union`"""
+
+    def __sub__(other):
+        """Shortcut for :meth:`~BTrees.Interfaces.IMerge.difference`"""
+
 
 class IMerge(Interface):
     """Object with methods for merging sets, buckets, and trees.
@@ -271,11 +376,15 @@ class IMerge(Interface):
     These methods are supplied in modules that define collection
     classes with particular key and value types. The operations apply
     only to collections from the same module.  For example, the
-    IIBTree.union can only be used with IIBTree.IIBTree,
-    IIBTree.IIBucket, IIBTree.IISet, and IIBTree.IITreeSet.
+    :meth:`BTrees.IIBTree.IIBTree.union` can only be used with :class:`~BTrees.IIBTree.IIBTree`,
+    :class:`~BTrees.IIBTree.IIBucket`, :class:`~BTrees.IIBTree.IISet`, and :class:`~BTrees.IIBTree.IITreeSet`.
 
-    The implementing module has a value type. The IOBTree and OOBTree
-    modules have object value type. The IIBTree and OIBTree modules
+    The number protocols methods ``__and__``, ``__or__`` and ``__sub__`` are provided
+    by all the data structures. They are shortcuts for :meth:`~BTrees.Interfaces.IMerge.intersection`,
+    :meth:`~BTrees.Interfaces.IMerge.union` and :meth:`~BTrees.Interfaces.IMerge.difference`.
+
+    The implementing module has a value type. The :class:`~BTrees.IOBTree.IOBTree` and :class:`~BTrees.OOBTree.OOBTree`
+    modules have object value type. The :class:`~BTrees.IIBTree.IIBTree` and :class:`~BTrees.OIBTree.OIBTree` modules
     have integer value types. Other modules may be defined in the
     future that have other value types.
 
@@ -291,6 +400,12 @@ class IMerge(Interface):
 
         If neither c1 nor c2 is None, the output is a Set if c1 is a Set or
         TreeSet, and is a Bucket if c1 is a Bucket or BTree.
+
+        While *c1* must be one of those types, *c2* can be any Python iterable
+        returning the correct types of objects.
+
+        .. versionchanged:: 4.8.0
+           Add support for *c2* to be an arbitrary iterable.
         """
 
     def union(c1, c2):
@@ -301,6 +416,12 @@ class IMerge(Interface):
 
         The output is a Set containing keys from the input
         collections.
+
+        *c1* and *c2* can be any Python iterables returning the
+        correct type of objects.
+
+        .. versionchanged:: 4.8.0
+           Add support for arbitrary iterables.
         """
 
     def intersection(c1, c2):
@@ -311,6 +432,12 @@ class IMerge(Interface):
 
         The output is a Set containing matching keys from the input
         collections.
+
+        *c1* and *c2* can be any Python iterables returning the
+        correct type of objects.
+
+        .. versionchanged:: 4.8.0
+           Add support for arbitrary iterables.
         """
 
 
@@ -418,9 +545,9 @@ class IIMerge(IMerge):
 
 
 class IMergeIntegerKey(IMerge):
-    """IMerge-able objects with integer keys.
+    """:class:`~BTrees.Interfaces.IMerge`-able objects with integer keys.
 
-    Concretely, this means the types in IOBTree and IIBTree.
+    Concretely, this means the types in :class:`~BTree.IOBTree.IOBTree` and :class:`~BTrees.IIBTree.IIBTree`.
     """
 
     def multiunion(seq):
@@ -432,59 +559,103 @@ class IMergeIntegerKey(IMerge):
         + An integer, which is added to the union.
 
         + A Set or TreeSet from the same module (for example, an
-          IIBTree.TreeSet for IIBTree.multiunion()).  The elements of the
+          :class:`BTrees.IIBTree.TreeSet` for :meth:`BTrees.IIBTree.multiunion`).  The elements of the
           set are added to the union.
 
         + A Bucket or BTree from the same module (for example, an
-          IOBTree.IOBTree for IOBTree.multiunion()).  The keys of the
+          :class:`BTrees.IOBTree.IOBTree` for :meth:`BTrees.IOBTree.multiunion`).  The keys of the
           mapping are added to the union.
 
+        + Any iterable Python object that iterates across integers. This
+          will be slower than the above types.
+
         The union is returned as a Set from the same module (for example,
-        IIBTree.multiunion() returns an IIBTree.IISet).
+        :meth:`BTrees.IIBTree.multiunion` returns an :class:`BTrees.IIBTree.IISet`).
 
         The point to this method is that it can run much faster than
-        doing a sequence of two-input union() calls.  Under the covers,
+        doing a sequence of two-input :meth:`~BTrees.Interfaces.IMerge.union` calls.  Under the covers,
         all the integers in all the inputs are sorted via a single
         linear-time radix sort, then duplicates are removed in a second
         linear-time pass.
+
+        .. versionchanged:: 4.8.0
+           Add support for arbitrary iterables of integers.
         """
 
 class IBTreeFamily(Interface):
     """the 64-bit or 32-bit family"""
-    IO = Attribute('The IIntegerObjectBTreeModule for this family')
-    OI = Attribute('The IObjectIntegerBTreeModule for this family')
-    II = Attribute('The IIntegerIntegerBTreeModule for this family')
     IF = Attribute('The IIntegerFloatBTreeModule for this family')
+    II = Attribute('The IIntegerIntegerBTreeModule for this family')
+    IO = Attribute('The IIntegerObjectBTreeModule for this family')
+    IU = Attribute('The IIntegerUnsignedBTreeModule for this family')
+
+    UF = Attribute('The IUnsignedFloatBTreeModule for this family')
+    UI = Attribute('The IUnsignedIntegerBTreeModule for this family')
+    UO = Attribute('The IUnsignedObjectBTreeModule for this family')
+    UU = Attribute('The IUnsignedUnsignedBTreeModule for this family')
+
+    OI = Attribute('The IObjectIntegerBTreeModule for this family')
     OO = Attribute('The IObjectObjectBTreeModule for this family')
-    maxint = Attribute('The maximum integer storable in this family')
-    minint = Attribute('The minimum integer storable in this family')
+    OU = Attribute('The IObjectUnsignedBTreeModule for this family')
+
+    maxint = Attribute('The maximum signed integer storable in this family')
+    maxuint = Attribute('The maximum unsigned integer storable in this family')
+    minint = Attribute('The minimum signed integer storable in this family')
 
 
-class IIntegerObjectBTreeModule(IBTreeModule, IMerge):
-    """keys, or set values, are integers; values are objects.
-
-    describes IOBTree and LOBTree"""
-
+class _IMergeBTreeModule(IBTreeModule, IMerge):
     family = Attribute('The IBTreeFamily of this module')
 
 
-class IObjectIntegerBTreeModule(IBTreeModule, IIMerge):
-    """keys, or set values, are objects; values are integers.
+class IIntegerObjectBTreeModule(_IMergeBTreeModule):
+    """keys, or set values, are signed integers; values are objects.
+
+    describes IOBTree and LOBTree"""
+
+
+class IUnsignedObjectBTreeModule(_IMergeBTreeModule):
+    """
+    As for :class:`~BTrees.Interfaces.IIntegerObjectBTreeModule` with unsigned integers.
+    """
+
+
+class IObjectIntegerBTreeModule(_IMergeBTreeModule):
+    """keys, or set values, are objects; values are signed integers.
 
     Object keys (and set values) must sort reliably (for instance, *not* on
     object id)!  Homogenous key types recommended.
 
     describes OIBTree and LOBTree"""
 
-    family = Attribute('The IBTreeFamily of this module')
+
+class IObjectUnsignedBTreeModule(_IMergeBTreeModule):
+    """
+    As for `IObjectIntegerBTreeModule` with unsigned integers.
+    """
 
 
-class IIntegerIntegerBTreeModule(IBTreeModule, IIMerge, IMergeIntegerKey):
-    """keys, or set values, are integers; values are also integers.
+class IIntegerIntegerBTreeModule(_IMergeBTreeModule, IMergeIntegerKey):
+    """keys, or set values, are signed integers; values are also signed integers.
 
     describes IIBTree and LLBTree"""
 
-    family = Attribute('The IBTreeFamily of this module')
+
+class IUnsignedUnsignedBTreeModule(_IMergeBTreeModule, IMergeIntegerKey):
+    """
+    As for `IIntegerIntegerBTreeModule` with unsigned integers.
+    """
+
+
+class IUnsignedIntegerBTreeModule(_IMergeBTreeModule, IMergeIntegerKey):
+    """
+    As for `IIntegerIntegerBTreeModule` with unsigned integers for keys only.
+    """
+
+
+class IIntegerUnsignedBTreeModule(_IMergeBTreeModule, IMergeIntegerKey):
+    """
+    As for `IIntegerIntegerBTreeModule` with unsigned integers for values only.
+    """
 
 
 class IObjectObjectBTreeModule(IBTreeModule, IMerge):
@@ -499,12 +670,15 @@ class IObjectObjectBTreeModule(IBTreeModule, IMerge):
     # the OO flavor of BTrees.
 
 
-class IIntegerFloatBTreeModule(IBTreeModule, IMerge):
-    """keys, or set values, are integers; values are floats.
+class IIntegerFloatBTreeModule(_IMergeBTreeModule):
+    """keys, or set values, are signed integers; values are floats.
 
     describes IFBTree and LFBTree"""
 
-    family = Attribute('The IBTreeFamily of this module')
+class IUnsignedFloatBTreeModule(_IMergeBTreeModule):
+    """
+    As for `IIntegerFloatBTreeModule` with unsigned integers.
+    """
 
 
 try:
